@@ -122,9 +122,15 @@ bbcode_expressions: list[tuple[Pattern, str]] = [
     (re_compile(r"\[url=([^]]+)]"), r'<a href="\1">'),
     (re_compile(r"\[/url]"), "</a>"),
     (re_compile(r"\[yt]((?:.(?!\[yt]))+)\[/yt]"), r'</a href="\1">\1</a>'),
-    (re_compile(r"(:icon([^:]+):|:([^:]+)icon:)"), r'<a href="/user/\2\3">@\2\3</a>'),
-    (re_compile(r"((?:\n(?!\n)|[^\n])+)\n\n+"), r"<p>\1</p>"),
-    (re_compile(r"\n"), "<br/>")
+    (re_compile(r"(:(?:icon|link)([^:]+):|:([^:]+)icon:)"), r'<a href="/user/\2\3">@\2\3</a>'),
+    (re_compile(r"@(\S+)"), r'<a href="/user/\1">@\1</a>'),
+    (re_compile(r"\n"), "<br/>"),
+    (re_compile(r"[-=]{5,}"), "<hr/>"),
+    (re_compile(r"\[ *(\d*), *(\d*) *, *(\d*) *]"),
+     r'<a href="/submission/\1">PREV</a>|<a href="/submission/\2">FIRST</a>|<a href="/submission/\3">NEXT</a>'),
+    (re_compile(r"\(c\)"), "&copy;"),
+    (re_compile(r"\(r\)"), "&reg;"),
+    (re_compile(r"\(tm\)"), "&trade;"),
 ]
 
 app.mount("/static", StaticFiles(directory=settings.static_folder), "static")
@@ -146,7 +152,7 @@ def prepare_comments(comments: list[dict]) -> list[dict]:
 
 
 def bbcode_to_html(text: str) -> str:
-    return reduce(lambda t, es: es[0].sub(es[1], t), bbcode_expressions, text.strip() + "\n\n")
+    return "".join(reduce(lambda txt, exp: exp[0].sub(exp[1], txt), bbcode_expressions, text.strip()))
 
 
 def clean_html(html: str) -> str:
@@ -213,14 +219,14 @@ def log_settings():
     logger.info(f"Using SSL certificate: {settings.ssl_cert}") if settings.ssl_cert else None
     logger.info(f"Using SSL private key: {settings.ssl_key}") if settings.ssl_key else None
     logger.info(f"Using HTTP Basic authentication") if settings.username or settings.password else None
+    settings.database.clear_cache(settings.database.m_time)
     if settings.precache:
         for table, order in [
             (t, o)
             for t in (settings.database.users, settings.database.submissions, settings.database.journals)
             for o in ("asc", "desc")
         ]:
-            logger.info(
-                f"Caching {table.name.upper()}:{(sort := search_settings.sort[table.name]).upper()}:{order.upper()}")
+            logger.info("Caching " + f"{table.name}:{(sort := search_settings.sort[table.name])}:{order}".upper())
             settings.database.load_search(table.name, "", "id" if sort.lower() == "date" else sort, order)
     if settings.open_browser:
         open_browser(settings.address)
@@ -487,8 +493,10 @@ async def serve_submission(request: Request, id_: int):
         "app": app.title,
         "title": f"{sub['TITLE']} by {sub['AUTHOR']}",
         "submission": sub | {"DESCRIPTION": clean_html(sub["DESCRIPTION"])},
-        "files_text": [bbcode_to_html(fs[i].read_text(detect_encoding(fs[i].read_bytes())["encoding"], "ignore"))
-                       for i, ext in enumerate(sub['FILEEXT']) if ext.lower() == "txt"] if fs else [],
+        "files_text": [
+            bbcode_to_html(fs[i].read_text(detect_encoding(fs[i].read_bytes())["encoding"], "ignore"))
+            if ext.lower() == "txt" else ""
+            for i, ext in enumerate(sub['FILEEXT'])] if fs else [],
         "filenames": [f"submission{('.' + ext) * bool(ext)}" for ext in sub['FILEEXT']],
         "filenames_id": [f"{sub['ID']:010d}{('.' + ext) * bool(ext)}" for ext in sub['FILEEXT']],
         "comments": prepare_comments(settings.database.load_submission_comments(id_)),
@@ -681,8 +689,7 @@ def run_redirect(host: str, port_listen: int, port_redirect: int):
         lambda r, *_: RedirectResponse(f"https://{r.url.hostname}:{port_redirect}{r.url.path}?{r.url.query}"),
         ["GET"]
     )
-    # noinspection PyTypeChecker
-    run(redirect_app, host=host, port=port_listen)
+    run(redirect_app, host=host, port=port_listen, log_config=LOGGING_CONFIG)
 
 
 def server(database_path: str | PathLike, host: str = "0.0.0.0", port: int = None,
@@ -709,7 +716,6 @@ def server(database_path: str | PathLike, host: str = "0.0.0.0", port: int = Non
                        f"{'localhost' if host == '0.0.0.0' else host}" \
                        f"{f':{port}' if port else ''}"
     run_args |= {"port": run_args.get("port", port) or 80}
-    with Database(Path(database_path).resolve()) as settings.database:
+    with Database(Path(database_path).resolve(), logger) as settings.database:
         search_settings.load(settings.database.load_settings("SEARCH"))
-        # noinspection PyTypeChecker
-        run(app, host=host, **run_args)
+        run(app, host=host, **run_args, log_config=LOGGING_CONFIG)
